import 'package:danger_core/src/mock_json_annotation.dart';
import 'package:danger_core/src/models/git_dsl.dart';

part 'github_dsl.g.dart';

enum GitHubPRState {
  closed,
  open,
  locked,
  merged,
}

enum GitHubUserType {
  user,
  organization,
  bot,
}

enum GitHubPRAuthorAssociation {
  collaborator,
  contributor,
  firstTimer,
  firstTimeContributor,
  member,
  none,
  owner,
}

enum GitHubReviewState {
  approved,
  changesRequested,
  dimissed,
  requestChanges,
  comment,
  pending,
  commented,
}

class GitHubDSL {
  /// The issue metadata for a code review session
  final GitHubIssue issue;

  /// The PR metadata for a code review session
  final GitHubPRDSL pr;

  /// The PR metadata specifically formatted for using with the GitHub API client
  final GitHubAPIPR thisPR;

  /// The github commit metadata for a code review session
  final List<GitHubCommit> commits;

  /// The reviews left on this pull request
  final List<GitHubReview> reviews;

  /// The people/teams requested to review this PR
  final GitHubReviewers requestedReviewers;

  GitHubDSL(
      {required this.issue,
      required this.pr,
      required this.thisPR,
      required this.commits,
      required this.reviews,
      required this.requestedReviewers});

  factory GitHubDSL.fromJson(Map<String, dynamic> json) =>
      _$GitHubDSLFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubDSLToJson(this);
}

class GitHubPRDSL {
  /// The UUID for the PR
  final int number;

  /// The state for the PR
  final GitHubPRState state;

  /// Has the PR been locked to contributors only?
  final bool locked;

  /// The title of the PR
  final String title;

  /// The markdown body message of the PR
  final String? body;

  /// ISO6801 Date string for when PR was created
  final String createdAt;

  /// ISO6801 Date string for when PR was updated
  final String updatedAt;

  /// optional ISO6801 Date string for when PR was closed
  final String? closedAt;

  /// Optional ISO6801 Date string for when PR was merged.
  final String? mergedAt;

  /// Merge reference for the _other_ repo.
  final GitHubMergeRef head;

  /// Merge reference for _this_ repo.
  final GitHubMergeRef base;

  /// The User who submitted the PR
  final GitHubUser user;

  /// The User who is assigned the PR
  final GitHubUser? assignee;

  /// The Users who are assigned to the PR
  final List<GitHubUser>? assignees;

  /// Has the PR been merged yet?
  final bool? merged;

  /// The number of comments on the PR
  final int comments;

  /// The number of review-specific comments on the PR
  final int reviewComments;

  /// The number of commits in the PR
  final int commits;

  /// The number of additional lines in the PR
  final int? additions;

  /// The number of deleted lines in the PR
  final int? deletions;

  /// The number of changed files in the PR
  final int? changedFiles;

  /// The link back to this PR as user-facing
  final String htmlUrl;

  /// How does the PR author relate to this repo/org?
  final GitHubPRAuthorAssociation? authorAssociation;

  GitHubPRDSL(
      {required this.number,
      required this.state,
      required this.locked,
      required this.title,
      this.body,
      required this.createdAt,
      required this.updatedAt,
      this.closedAt,
      this.mergedAt,
      required this.head,
      required this.base,
      required this.user,
      this.assignee,
      this.assignees,
      this.merged,
      required this.comments,
      required this.reviewComments,
      required this.commits,
      this.additions,
      this.deletions,
      this.changedFiles,
      required this.htmlUrl,
      this.authorAssociation});

  factory GitHubPRDSL.fromJson(Map<String, dynamic> json) =>
      _$GitHubPRDSLFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubPRDSLToJson(this);
}

class GitHubIssue {
  /// The labels associated with this issue
  final List<GitHubIssueLabel> labels;

  GitHubIssue({required this.labels});

  factory GitHubIssue.fromJson(Map<String, dynamic> json) =>
      _$GitHubIssueFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubIssueToJson(this);
}

class GitHubIssueLabel {
  /// The identifying number of this label
  final int id;

  /// The URL that links to this label
  final String url;

  /// The name of the label
  final String name;

  /// The color associated with this label
  final String color;

  GitHubIssueLabel(
      {required this.id,
      required this.url,
      required this.name,
      required this.color});

  factory GitHubIssueLabel.fromJson(Map<String, dynamic> json) =>
      _$GitHubIssueLabelFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubIssueLabelToJson(this);
}

class GitHubUser {
  /// Generic UUID
  final int id;

  /// The handle for the user/org
  final String login;

  /// Whether the user is an org, or a user
  final GitHubUserType type;

  ///The url for a users's image
  final String? avatarUrl;

  ///The href for a users's page
  final String href;

  GitHubUser(
      {required this.id,
      required this.login,
      required this.type,
      this.avatarUrl,
      required this.href});

  factory GitHubUser.fromJson(Map<String, dynamic> json) =>
      _$GitHubUserFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubUserToJson(this);
}

class GitHubRepo {
  /// Generic UUID
  final int id;

  /// The name of the repo, e.g. "Danger-JS"
  final String name;

  /// The full name of the owner + repo, e.g. "Danger/Danger-JS"
  final String fullName;

  /// The owner of the repo
  final GitHubUser owner;

  /// Is the repo publicly accessible?
  final bool private;

  /// The textual description of the repo
  final String? description;

  /// Is the repo a fork?
  final bool fork;

  /// The root web URL for the repo, e.g. https://github.com/artsy/emission
  final String htmlUrl;

  GitHubRepo(
      {required this.id,
      required this.name,
      required this.fullName,
      required this.owner,
      required this.private,
      this.description,
      required this.fork,
      required this.htmlUrl});

  factory GitHubRepo.fromJson(Map<String, dynamic> json) =>
      _$GitHubRepoFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubRepoToJson(this);
}

class GitHubMergeRef {
  /// The human display name for the merge reference, e.g. "artsy:master"
  final String label;

  /// The reference point for the merge, e.g. "master"
  final String ref;

  /// The reference point for the merge, e.g. "704dc55988c6996f69b6873c2424be7d1de67bbe"
  final String sha;

  /// The user that owns the merge reference e.g. "artsy"
  final GitHubUser user;

  /// The repo from whch the reference comes from
  final GitHubRepo repo;

  GitHubMergeRef(
      {required this.label,
      required this.ref,
      required this.sha,
      required this.user,
      required this.repo});

  factory GitHubMergeRef.fromJson(Map<String, dynamic> json) =>
      _$GitHubMergeRefFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubMergeRefToJson(this);
}

class GitHubAPIPR {
  /// The repo owner
  final String owner;

  /// The repo name
  final String repo;

  /// The PR number
  final int number;

  GitHubAPIPR({required this.owner, required this.repo, required this.number});

  factory GitHubAPIPR.fromJson(Map<String, dynamic> json) =>
      _$GitHubAPIPRFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubAPIPRToJson(this);
}

class GitHubCommit {
  /// The raw commit metadata
  final GitCommit commit;

  /// The SHA for the commit
  final String sha;

  /// the url for the commit on GitHub
  final String url;

  /// The GitHub user who wrote the code
  final GitHubUser? author;

  /// The GitHub user who shipped the code
  final GitHubUser? committer;

  /// An array of parent commit shas
  final List<dynamic>? parents;

  GitHubCommit(
      {required this.commit,
      required this.sha,
      required this.url,
      this.author,
      this.committer,
      this.parents});

  factory GitHubCommit.fromJson(Map<String, dynamic> json) =>
      _$GitHubCommitFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubCommitToJson(this);
}

class GitHubReview {
  /// The user requested to review, or the user who has completed the review
  final GitHubUser user;

  /// If there is a review, this provides the ID for it
  final int? id;

  /// If there is a review, the body of the review
  final String? body;

  /// If there is a review, the commit ID this review was made on
  final String? commitId;

  /// The state of the review
  /// * APPROVED, REQUEST_CHANGES, COMMENT or PENDING
  final GitHubReviewState? state;

  GitHubReview(
      {required this.user, this.id, this.body, this.commitId, this.state});

  factory GitHubReview.fromJson(Map<String, dynamic> json) =>
      _$GitHubReviewFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubReviewToJson(this);
}

class GitHubReviewers {
  /// Users that have been requested
  final List<GitHubUser> users;

  /// Teams that have been requested
  final List<dynamic> teams;

  GitHubReviewers({required this.users, required this.teams});

  factory GitHubReviewers.fromJson(Map<String, dynamic> json) =>
      _$GitHubReviewersFromJson(json);
  Map<String, dynamic> toJson() => _$GitHubReviewersToJson(this);
}